Set Warnings "-notation-overridden,-parsing".
Require Import Sets.
Parameter MISSING: Type.   

Module Check.  

Ltac check_type A B :=  
match type of A with  
| context[MISSING] => idtac "Missing:" A  
| ?T => first [unify T B; idtac "Type: ok" | idtac "Type: wrong - should be (" B ")"]  
end.  

Ltac print_manual_grade A :=  
first [  
match eval compute in A with  
| ?T => idtac "Score:" T  
end  
| idtac "Score: Ungraded"].  

End Check.

Require Import Sets.
Import Check.

Goal True.

idtac "-------------------  complement__universe_difference  --------------------".
idtac " ".

idtac "#> AxiomaticSets.complement__universe_difference".
idtac "Possible points: 1".
check_type @AxiomaticSets.complement__universe_difference (
(forall (X : Type) (S : AxiomaticSets.set X),
 @AxiomaticSets.Complement X S =
 @AxiomaticSets.Difference X (@AxiomaticSets.Universe X) S)).
idtac "Assumptions:".
Abort.
Print Assumptions AxiomaticSets.complement__universe_difference.
Goal True.
idtac " ".

idtac "-------------------  subset_power_set  --------------------".
idtac " ".

idtac "#> AxiomaticSets.subset_power_set".
idtac "Possible points: 1".
check_type @AxiomaticSets.subset_power_set (
(forall (X : Type) (S1 S2 : AxiomaticSets.set X),
 @AxiomaticSets.subset X S1 S2 ->
 @AxiomaticSets.subset (AxiomaticSets.set X) (@AxiomaticSets.power_set X S1)
   (@AxiomaticSets.power_set X S2))).
idtac "Assumptions:".
Abort.
Print Assumptions AxiomaticSets.subset_power_set.
Goal True.
idtac " ".

idtac "-------------------  finite_sets  --------------------".
idtac " ".

idtac "#> Manually graded: finite_sets".
idtac "Possible points: 1".
print_manual_grade score_finite_sets.
idtac " ".

idtac "-------------------  de_morgan2  --------------------".
idtac " ".

idtac "#> AxiomaticSets.de_morgan2".
idtac "Possible points: 2".
check_type @AxiomaticSets.de_morgan2 (
(forall (X : Type) (S1 S2 : AxiomaticSets.set X),
 @AxiomaticSets.Complement X
   (@AxiomaticSets.Union X (@AxiomaticSets.Complement X S1)
      (@AxiomaticSets.Complement X S2)) = @AxiomaticSets.Intersection X S1 S2)).
idtac "Assumptions:".
Abort.
Print Assumptions AxiomaticSets.de_morgan2.
Goal True.
idtac " ".

idtac "-------------------  intersect_empty_r  --------------------".
idtac " ".

idtac "#> Manually graded: intersect_empty_r".
idtac "Possible points: 2".
print_manual_grade score_intersect_empty_r.
idtac " ".

idtac "-------------------  union_algebraic  --------------------".
idtac " ".

idtac "#> Manually graded: union_algebraic".
idtac "Possible points: 2".
print_manual_grade score_union_algebraic.
idtac " ".

idtac "-------------------  R_functional  --------------------".
idtac " ".

idtac "#> AxiomaticSets.R_functional".
idtac "Possible points: 2".
check_type @AxiomaticSets.R_functional (
(@AxiomaticSets.functional nat nat AxiomaticSets.R)).
idtac "Assumptions:".
Abort.
Print Assumptions AxiomaticSets.R_functional.
Goal True.
idtac " ".

idtac "-------------------  R_fR  --------------------".
idtac " ".

idtac "#> AxiomaticSets.R_fR".
idtac "Possible points: 1".
check_type @AxiomaticSets.R_fR (
(forall a b : nat,
 @AxiomaticSets.related_in nat nat AxiomaticSets.R a b ->
 AxiomaticSets.fR a = b)).
idtac "Assumptions:".
Abort.
Print Assumptions AxiomaticSets.R_fR.
Goal True.
idtac " ".

idtac "-------------------  R_converse  --------------------".
idtac " ".

idtac "#> Manually graded: R_converse".
idtac "Possible points: 1".
print_manual_grade score_R_converse.
idtac " ".

idtac "-------------------  city_names  --------------------".
idtac " ".

idtac "#> Manually graded: city_names".
idtac "Possible points: 1".
print_manual_grade score_city_names.
idtac " ".

idtac "-------------------  broken_proof  --------------------".
idtac " ".

idtac "#> Manually graded: broken_proof".
idtac "Possible points: 1".
print_manual_grade score_broken_proof.
idtac " ".

idtac "-------------------  lifted_functions  --------------------".
idtac " ".

idtac "#> Manually graded: lifted_functions".
idtac "Possible points: 3".
print_manual_grade score_lifted_functions.
idtac " ".

idtac "-------------------  inj_composition  --------------------".
idtac " ".

idtac "#> AxiomaticSets.inj_composition".
idtac "Possible points: 1".
check_type @AxiomaticSets.inj_composition (
(forall (A B C : Type) (f : A -> B) (g : B -> C),
 @AxiomaticSets.injective A B f ->
 @AxiomaticSets.injective B C g ->
 @AxiomaticSets.injective A C (@AxiomaticSets.compose A B C f g))).
idtac "Assumptions:".
Abort.
Print Assumptions AxiomaticSets.inj_composition.
Goal True.
idtac " ".

idtac "-------------------  surj_composition  --------------------".
idtac " ".

idtac "#> AxiomaticSets.surj_composition".
idtac "Possible points: 1".
check_type @AxiomaticSets.surj_composition (
(forall (A B C : Type) (f : A -> B) (g : B -> C),
 @AxiomaticSets.surjective A B f ->
 @AxiomaticSets.surjective B C g ->
 @AxiomaticSets.surjective A C (@AxiomaticSets.compose A B C f g))).
idtac "Assumptions:".
Abort.
Print Assumptions AxiomaticSets.surj_composition.
Goal True.
idtac " ".

idtac "-------------------  nat_natopt_bij  --------------------".
idtac " ".

idtac "#> AxiomaticSets.nat_natopt_bij".
idtac "Possible points: 2".
check_type @AxiomaticSets.nat_natopt_bij ((nat -> option nat)).
idtac "Assumptions:".
Abort.
Print Assumptions AxiomaticSets.nat_natopt_bij.
Goal True.
idtac " ".

idtac "-------------------  bool_inj__surj  --------------------".
idtac " ".

idtac "#> AxiomaticSets.bool_inj__surj".
idtac "Possible points: 3".
check_type @AxiomaticSets.bool_inj__surj (
(forall f : bool -> bool,
 @AxiomaticSets.injective bool bool f <->
 @AxiomaticSets.surjective bool bool f)).
idtac "Assumptions:".
Abort.
Print Assumptions AxiomaticSets.bool_inj__surj.
Goal True.
idtac " ".

idtac "-------------------  bool_nat_cardinality  --------------------".
idtac " ".

idtac "#> AxiomaticSets.bool_nat_cardinality".
idtac "Possible points: 2".
check_type @AxiomaticSets.bool_nat_cardinality (
(~ AxiomaticSets.same_cardinality bool nat)).
idtac "Assumptions:".
Abort.
Print Assumptions AxiomaticSets.bool_nat_cardinality.
Goal True.
idtac " ".

idtac "-------------------  set_powset_diag  --------------------".
idtac " ".

idtac "#> Manually graded: set_powset_diag".
idtac "Possible points: 4".
print_manual_grade score_set_powset_diag.
idtac " ".

idtac "-------------------  evens_nat  --------------------".
idtac " ".

idtac "#> Manually graded: evens_nat".
idtac "Possible points: 3".
print_manual_grade score_evens_nat.
idtac " ".

idtac "-------------------  countable_union  --------------------".
idtac " ".

idtac "#> Manually graded: countable_union".
idtac "Possible points: 3".
print_manual_grade score_countable_union.
idtac " ".

idtac "-------------------  countable_product  --------------------".
idtac " ".

idtac "#> Manually graded: countable_product".
idtac "Possible points: 4".
print_manual_grade score_countable_product.
idtac " ".

idtac "-------------------  uncountable_complex  --------------------".
idtac " ".

idtac "#> Manually graded: uncountable_complex".
idtac "Possible points: 2".
print_manual_grade score_uncountable_complex.
idtac " ".

idtac "-------------------  uncountable_product  --------------------".
idtac " ".

idtac "#> Manually graded: uncountable_product".
idtac "Possible points: 2".
print_manual_grade score_uncountable_product.
idtac " ".

idtac " ".

idtac "Max points - standard: 45".
idtac "Max points - advanced: 45".
Abort.
